package de.hsrm.cs.javaee8.statelessmaven.ejb;

import java.util.logging.Logger;

import javax.ejb.Stateless;

/**
 * This is the Stateless SessionBean "GeometricModel", which might be used for
 * simple geometric calculations.
 * 
 */
@Stateless()
public class GeometricModelBean implements GeometricModelLocal
{
  /** The logger. It is a static constant (static variables have to be final !) */
  protected static final Logger logger = Logger.getLogger(GeometricModelBean.class.getName());

  /**
   * Constructor.
   */
  public GeometricModelBean()
  {
  }

  // /////////////////////////////////////////////////////////////////
  // Bussiness Methods
  // /////////////////////////////////////////////////////////////////

  /**
   * Compute the volume of a square stone
   * 
   * @param a Length of side a, &gt; 0
   * @param b Length of side b, &gt; 0
   * @param c Length of side c, &gt; 0
   * @return Volume of the square stone = a * b * c
   * @exception IllegalArgumentException If a or b or c &lt;= 0.
   */
  public double computeCuboidVolume(double a, double b, double c) throws IllegalArgumentException
  {
    logger.info("computeCuboidVolume with a = " + a + ", b = " + b + ", c = " + c);
    if (a <= 0 || b <= 0 || c <= 0)
      throw new IllegalArgumentException("Side length <= 0");

    return a * b * c;
  }

  /**
   * Compute the surface of a square stone
   * 
   * @param a Length of side a, &gt; 0
   * @param b Length of side b, &gt; 0
   * @param c Length of side c, &gt; 0
   * @return Surface of the square stone = 2 * (a * b) + 2 * (b*c) + 2 * (a * c)
   * @exception IllegalArgumentException If a or b or c &lt;= 0.
   */
  public double computeCuboidSurface(double a, double b, double c) throws IllegalArgumentException
  {
    logger.info("computeCuboidSurface with a = " + a + ", b = " + b + ", c = " + c);
    if (a <= 0 || b <= 0 || c <= 0)
      throw new IllegalArgumentException("Side length <= 0");

    return 2 * (a * b + b * c + c * a);
  }
}
