package de.hsrm.cs.javaee8.statelessmaven.web.test;

import java.io.File;

import javax.ejb.EJB;

import org.jboss.arquillian.container.test.api.Deployment;
import org.jboss.arquillian.junit.Arquillian;
import org.jboss.shrinkwrap.api.Archive;
import org.jboss.shrinkwrap.api.ShrinkWrap;
import org.jboss.shrinkwrap.api.asset.StringAsset;
import org.jboss.shrinkwrap.api.importer.ZipImporter;
import org.jboss.shrinkwrap.api.spec.EnterpriseArchive;
import org.jboss.shrinkwrap.api.spec.JavaArchive;
import org.jboss.shrinkwrap.api.spec.WebArchive;
import org.jboss.shrinkwrap.descriptor.api.Descriptors;
import org.jboss.shrinkwrap.descriptor.api.application7.ApplicationDescriptor;
import org.junit.Assert;
import org.junit.Test;
import org.junit.runner.RunWith;

import de.hsrm.cs.javaee8.statelessmaven.ejb.GeometricModelLocal;

/**Unit Test
 * 
 * @author Wolfgang Knauf
 *
 */
@RunWith(Arquillian.class)
public class GeometricModelBeanIT
{
  /**Local interface for Bean @see GeometricModelBean is injected by the server.
   */
  @EJB
  private GeometricModelLocal geometricModelLocal;

  /**Liefert das EAR-Deployment, das auf den Server gepackt werden soll.
   * 
   * @return EAR-Archiv
   */
  @Deployment
  public static Archive<?> getEarArchive() 
  {
    EnterpriseArchive ear = ShrinkWrap.create(EnterpriseArchive.class, "StatelessMaven-ear.ear");
    
    //Wir stehen aktuell im Verzeichnis des "StatelessMaven-web"-Projekts. Also eine Ebene höher gehen und in das "StatelessMaven-ejb"-Projekt gehen.
    
    File f = new File("../StatelessMaven-ejb/target/StatelessMaven-ejb.jar");
    System.err.println("Exists: " + f.getAbsolutePath() + "=>" + f.exists());
    JavaArchive ejbJar = ShrinkWrap.create(ZipImporter.class, "StatelessMaven-ejb.jar").
          importFrom(f).as(JavaArchive.class) ;
    ear.addAsModule(ejbJar);
     
    
    f = new File("../StatelessMaven-web/target/StatelessMaven-web.war");
    if (f.exists() == false)
    {
      System.err.println("File " + f.getAbsolutePath()  + " does not exist.");
      //Exception-Message geht leider verloren...
      throw new RuntimeException("File " + f.getAbsolutePath()  + " does not exist.");
    }
    System.err.println("Exists: " + f.getAbsolutePath() + "=>" + f.exists());
    WebArchive war = ShrinkWrap.create(ZipImporter.class, "StatelessMaven-web.war").
          importFrom(f).as(WebArchive.class) ;
    ear.addAsModule(war);
     
    //"application.xml" händisch aufbauen mit Hilfe des "Shrinkwrap Descriptors"-Projekts:
    ApplicationDescriptor descriptor = Descriptors.create(ApplicationDescriptor.class, "StatelessMaven-ear");
    descriptor.applicationName("StatelessMaven-ear");
    
    //Webmodul bauen:
    descriptor.createModule().getOrCreateWeb().contextRoot("/StatelessMaven-web").webUri("StatelessMaven-web.war");
    
    //EJB-Modul reinpacken:
    descriptor.createModule().ejb("StatelessMaven-ejb.jar");
    
    //In EAR-Datei schreiben:
    ear.setApplicationXML(new StringAsset(descriptor.exportAsString()));
    
    
    //Manifest-Datei haben wir nicht.
    
    //Packages mit Test-Klassen zum Deployment zufügen:
    war.addPackage("de.hsrm.cs.javaee8.statelessmaven.web.test");
      
    //EAR exportieren zum Anschauen:
    //ear.as(org.jboss.shrinkwrap.api.exporter.ZipExporter.class).exportTo(new File("c:\\temp\\test.ear"), true);
   
    return ear;
  }
  
  /**Test durchführen: Oberflächenberechnung 
   * 
   */
  @Test
  public void testCuboidSurface()
  {
    System.out.println("Testing cuboid surface...");
    
    /*
    //Zum Debuggen: lange warten, damit man sich Details anschauen kann, z.B. das auf den Server deployed Archiv.
    try
    {
      Thread.sleep(1000 * 60 * 5);
    }
    catch (InterruptedException e)
    {
      // TODO Auto-generated catch block
      e.printStackTrace();
    }
    System.out.println("end waiting...");
    */
    
    double surface = geometricModelLocal.computeCuboidSurface(1, 2, 3);
    //Hier muss ein "Delta" angegeben werden für Abweichung durch Rundungsungenauigkeit. Das kann hier nicht zuschlagen, deshalb können wir Delta = 0 angeben.
    Assert.assertEquals(22, surface, 0);
  }
  
  /**Test durchführen: Volumenberechnung 
   * 
   */
  @Test
  public void testCuboidVolume()
  {
    System.out.println("Testing cuboid volume...");
                
    double volume = geometricModelLocal.computeCuboidVolume(1, 2, 3);
    //Hier muss ein "Delta" angegeben werden für Abweichung durch Rundungsungenauigkeit. Das kann hier nicht zuschlagen, deshalb können wir Delta = 0 angeben.
    Assert.assertEquals(6, volume, 0);
  }
}